#!/usr/bin/perl -w
use strict;
use warnings;

###############################################################################################################
sub ConvertIUPAC {
	my($search_motif) = @_;
	my($motif_name) = "";

	$search_motif =~ s/^\s+//gi;
	$search_motif =~ s/\s+$//gi;
	$motif_name = $search_motif;
	$search_motif = uc($search_motif);
	$search_motif =~ s/M/[AC]/gi;
	$search_motif =~ s/R/[AG]/gi;
	$search_motif =~ s/W/[AT]/gi;
	$search_motif =~ s/S/[CG]/gi;
	$search_motif =~ s/Y/[CT]/gi;
	$search_motif =~ s/K/[GT]/gi;
	$search_motif =~ s/V/[ACG]/gi;
	$search_motif =~ s/H/[ACT]/gi;
	$search_motif =~ s/D/[AGT]/gi;
	$search_motif =~ s/B/[CGT]/gi;
	$search_motif =~ s/N/[ACGT]/gi;
	
	return($motif_name, $search_motif);
};
###############################################################################################################
sub ConvertMatrix{
	my($matrix_tmp, $hash_seq_ptr, $matrix_cutoff) = @_;
	my(@header) 			= ();
	my(@help_matrix) 		= ();
        my(@mat) 			= ([],[]);	#the matrix in matrix-form (Mij i = (ACGT) j = lenght)
        my(@maxmat) 			= ([]);		#max sum of the rest
	my($matrix) 			= "";
	my($motif_name) 		= "";
	my($hashlen) 			= 0;
	my($maximal_score) 		= 0;	
	
	open(IN, "<$matrix_tmp") or die "could not open file $matrix_tmp\n"; 
	for(<IN>) { $matrix .= $_;}; 
	close(IN);
        
	@header = split(/\n/, $matrix);             #matrix lines
	$motif_name = shift(@header);
	$motif_name =~ s/\n//g; 
	$motif_name =~ s/\r//g; 
	$hashlen = 0;
	
	for (my($i) = 0; $i < @header; ++$i) {
		my(@head) = split(/\s+/, $header[$i]);
		for (my($j) = 0; $j < scalar(@head); ++$j) {
			$mat[$j][$i] = $head[$j];
		}
	}

		
	#for (my($i) = 0; $i < 4; ++$i) {
	#        my(@head) = split(/\s+/, $header[$i]);
	#        for (my($j) = 0; $j < scalar(@head); ++$j) {
	#                $mat[$i][$j] = $head[$j];
	#        }
	#}

        for (my($i) = 0; $i < scalar(@mat); ++$i) {
                if (scalar(@{$mat[$i]}) > $hashlen) { $hashlen = scalar(@{$mat[$i]}); }
        }
        
	for(my($i) = 0; $i < ($hashlen); ++$i) {
                $help_matrix[$i] = $mat[0][$i];
                for(my($j) = 1; $j < 4; ++$j) {
                        if ($help_matrix[$i] < $mat[$j][$i]) { $help_matrix[$i] = $mat[$j][$i]; }
                }
        }
        
	for (my($i) = ($hashlen-1); $i >= 0; --$i) {
                $maximal_score = $maximal_score + $help_matrix[$i];
                $maxmat[$i] = $maximal_score;
        }
        
	GenerateMatrixHash(0, 0, 0, 0, ($hashlen - 1), $hash_seq_ptr, $matrix_cutoff, \@mat, \@maxmat);
        GenerateMatrixHash(0, 1, 0, 1, ($hashlen - 1), $hash_seq_ptr, $matrix_cutoff, \@mat, \@maxmat);
        GenerateMatrixHash(0, 2, 0, 2, ($hashlen - 1), $hash_seq_ptr, $matrix_cutoff, \@mat, \@maxmat);
        GenerateMatrixHash(0, 3, 0, 3, ($hashlen - 1), $hash_seq_ptr, $matrix_cutoff, \@mat, \@maxmat);
	return($motif_name, $hashlen);
};
###############################################################################################################
sub GenerateMatrixHash {
        my($score, $i, $j, $hash_pattern, $mat_dim, $hash_seq_ptr, $matrix_cutoff, $mat_ptr, $maxmat_ptr) = @_;
        my($maximal_score) = $score + @$maxmat_ptr[$j];

        if ($maximal_score >= $matrix_cutoff) {
                $score = $score + @$mat_ptr[$i]->[$j];

                if ( ($j < $mat_dim) ) { 
                        GenerateMatrixHash($score, 0, ($j + 1), $hash_pattern."0", $mat_dim, $hash_seq_ptr, $matrix_cutoff, $mat_ptr, $maxmat_ptr);
                        GenerateMatrixHash($score, 1, ($j + 1), $hash_pattern."1", $mat_dim, $hash_seq_ptr, $matrix_cutoff, $mat_ptr, $maxmat_ptr);
                        GenerateMatrixHash($score, 2, ($j + 1), $hash_pattern."2", $mat_dim, $hash_seq_ptr, $matrix_cutoff, $mat_ptr, $maxmat_ptr);
                        GenerateMatrixHash($score, 3, ($j + 1), $hash_pattern."3", $mat_dim, $hash_seq_ptr, $matrix_cutoff, $mat_ptr, $maxmat_ptr);
                } else {
			if($score >= $matrix_cutoff) {
                        	$hash_pattern =~ tr/0123/ACGT/;
                        	${$hash_seq_ptr}{$hash_pattern} = 0; 
        		} 
	       }
        }
};
###############################################################################################################
1;
