use strict;
use warnings;

########################################################################################################################################
# Calculates anova (parameter,

sub CalculateANOVA {
	my($input_num, $clust_num, $stat_array_ptr, $result_ptr) = @_;

	my($promo_bck_mot) 	= @$stat_array_ptr[1];			#Number of promoter with motif in background
	my($promo_in1_mot) 	= @$stat_array_ptr[15];			#Number of promoter with motif in cluster1
	my($promo_in2_mot) 	= @$stat_array_ptr[29];			#Number of promoter with motif in cluster2
	my($promo_bck) 		= @$stat_array_ptr[0];			#Number of promoter in background
	my($promo_in1) 		= @$stat_array_ptr[14];			#Number of promoter in cluster1
	my($promo_in2) 		= @$stat_array_ptr[28];			#Number of promoter in cluster2
	my($in1_stdev) 		= @$stat_array_ptr[21];			#Motif distribution in cluster1
	my($in2_stdev) 		= @$stat_array_ptr[35];			#Motif distribution in cluster2

	my(@total)		= ();					#Motif total
	my(@stdsum2) 		= ();					#Motif distribution

	$total[0] 		= @$stat_array_ptr[4];			#Motif total background
	$total[1] 		= @$stat_array_ptr[18];			#Motif total cluster1
	$total[2] 		= @$stat_array_ptr[32];			#Motif total cluster2
	$stdsum2[0] 		= @$stat_array_ptr[13];			#Motif distribution background
	$stdsum2[1] 		= @$stat_array_ptr[27];			#Motif distribution cluster1
	$stdsum2[2] 		= @$stat_array_ptr[41];			#Motif distribution cluster2

	my($anova) 		= 0;
	my($anovaA) 		= 0;
	my($anovaW) 		= 0;
	my($anovaS) 		= 0;
	my($anovaB) 		= 0;
	my($anovaT) 		= 0;

	for (my($i) = 0; $i < 9; ++$i) { @$result_ptr[$i] = "ND"; }

	if ($input_num == 2 && (($promo_bck_mot != 0) || ($promo_in1_mot != 0) || ($promo_in2_mot != 0))) {
		if ($promo_bck != 0 && $promo_in1 != 0 && $promo_in2 != 0) {
			if($in1_stdev != 0 && $in2_stdev != 0) {
				for (my($i) = 0; $i < 3; ++$i) {
					$anova =  ((($total[$i])**2)/$clust_num);
					$anovaA = ($anovaA + $anova);
					$anovaW = ($anovaW + ($stdsum2[$i] - $anova));
					$anovaS = ($anovaS + $total[$i]);
					$anovaB = ($anovaA - (($anovaS**2)/(3*$clust_num)));
					$anovaT = ($anovaB + $anovaW);
				}
	
        			@$result_ptr[0] = sprintf("%.2f", ($anovaB));						#between
				@$result_ptr[1] = 2;									#dfB
        			@$result_ptr[2] = sprintf("%.2f", ($anovaB / 2));					#MSB
        			@$result_ptr[4] = sprintf("%.2f", ($anovaW));						#within
        			@$result_ptr[5] = sprintf("%.0f", ((3*$clust_num)-3));					#dfW
        			@$result_ptr[6] = sprintf("%.2f", ($anovaW / @$result_ptr[5]));				#MSW
        			@$result_ptr[7] = sprintf("%.2f", ($anovaT));						#total
        			@$result_ptr[8] = sprintf("%.0f", ((3*$clust_num)-1));					#dfT
        			@$result_ptr[3] = sprintf("%.2f", (@$result_ptr[2] / @$result_ptr[6]));			#Fpara

			} 
		}
	}
};
########################################################################################################################################
# Calculates T-tes

sub CalculateTtest { 
	my($input_num, $clust_num, $stat_array_ptr, $result_ptr) = @_;

	my($bck_stdev) 		= @$stat_array_ptr[7];
	my($in1_stdev) 		= @$stat_array_ptr[21];
	my($in2_stdev) 		= @$stat_array_ptr[35];

	my($bck_mean) 		= @$stat_array_ptr[8];
	my($in1_mean) 		= @$stat_array_ptr[22];
	my($in2_mean) 		= @$stat_array_ptr[36];
	
	my($promo_bck)          = @$stat_array_ptr[0];                  #Number of promoter in background
	my($promo_in1)          = @$stat_array_ptr[14];                 #Number of promoter in cluster1
	my($promo_in2)          = @$stat_array_ptr[28];                 #Number of promoter in cluster2

	for (my($i) = 0; $i < 6; ++$i) { @$result_ptr[$i] = "ND"; }
	
	if (($input_num == 1 || $input_num == 2) && ($bck_stdev != 0 && $in1_stdev != 0) && ($promo_bck != 0 && $promo_in1 != 0)) {
		@$result_ptr[0] = Ttest($bck_stdev, $in1_stdev, $bck_mean, $in1_mean, $clust_num);
		@$result_ptr[1] = sprintf("%.0f", ($clust_num + $clust_num - 2));


	} if ($input_num == 2  && ($bck_stdev != 0 && $in2_stdev != 0) && ($promo_bck != 0 && $promo_in2 != 0)) {
		@$result_ptr[2] = Ttest($bck_stdev, $in2_stdev, $bck_mean, $in2_mean, $clust_num);
		@$result_ptr[3] = sprintf("%.0f", ($clust_num + $clust_num - 2));
	} if ($input_num == 2  && ($in1_stdev != 0 && $in2_stdev != 0) && ($promo_in1 != 0 && $promo_in2 != 0)) {
		@$result_ptr[4] = Ttest($in1_stdev, $in2_stdev, $in1_mean, $in2_mean, $clust_num);
		@$result_ptr[5] = sprintf("%.0f", ($clust_num + $clust_num - 2));
	}
};

########################################################################################################################################
# t-test
sub Ttest {
	my($stdev1, $stdev2, $mean1, $mean2, $clust_num) = @_;
	my($test_result) = 0;
	my($std1) = 0;
	my($std2) = 0;
	my($std_tot) = 0;

	$std1 = (($stdev1 ** 2) / $clust_num);
	$std2 = (($stdev2 ** 2) / $clust_num);
	$std_tot = sqrt(($std1+$std2));
	
	$test_result = (($mean1 - $mean2) / $std_tot);
	#if ($test_result < 0) { $test_result = $test_result * -1; }
	#Just to make equal to POCO!#
	$test_result = $test_result * -1;
	$test_result = sprintf("%.2f", ($test_result));
	return($test_result);
};
########################################################################################################################################
1;
