#!/usr/bin/perl -w
use strict;
use DBI;
use warnings;
use FindBin qw($Bin);

use lib "$Bin/";
use HashNoOverlap; 
use HashOverlap; 
use StringNoOverlap;  
use StringOverlap;   
use CreateSearchPatterns;

###############################################################################################################
#Main program (the interface)
sub MainProgram {
	my($cluster_input1, $cluster_input2, $number_of_inputs, $number_of_genes, $number_of_samples, $search_motif, $sequence_length, $background, $motif_type, $matrix_tmp, $matrix_cutoff, $overlap, $sql_user, $sql_pass, $sql_db, $output, $stat_ptr, $hash_seq_ptr) = @_;

	my(%BootstrapSums) 				= ();	#hash which contains the sums of repeats
	my(%search_info) 				= ();	#hash which contains pattern information
	my(%mysql_info) 				= ();	#hash which contains mysql parameters
	my(@ID0) 					= ();	#Background motif (plus and minus strand) sum 
	my(@ID1) 					= ();	#Cluster1 motif (plus and minus strand) sum
	my(@ID2) 					= ();	#Cluster2 motif (plus and minus strand) sum
	my(@output_array0) 				= ();	#Output is temporarily stored here  
	my(@output_array1) 				= ();	#Output is temporarily stored here 
	my(@output_array2) 				= (); 	#Output is temporarily stored here 
	my($output_ptr)					= "";	#pointer to output arrays
	my($motif_name)        				= ""; 	#Name of the motif
	my($hashlen)					= 0;

	srand;
	
        if ($overlap != 1) 		{ $overlap = 0; }
        if ($number_of_inputs > 2) 	{ $number_of_inputs = 2; }
        if ($number_of_inputs < 1) 	{ $number_of_inputs = 0; }
        if ($sequence_length > 3000) 	{ $sequence_length = 3000; }
        if ($sequence_length < 0) 	{ $sequence_length = 0; }
	for (my($i) = 0; $i < 42; ++$i) { push(@$stat_ptr, "N.D"); }

	if ($motif_type == 1) 		{ ($motif_name, $search_motif) = ConvertIUPAC($search_motif); }
	if ($motif_type == 2) 		{ ($motif_name, $hashlen) = ConvertMatrix($matrix_tmp, $hash_seq_ptr, $matrix_cutoff); }

	$search_info{'hashlen'} = $hashlen;
	$search_info{'search_motif'} = $search_motif;
	$search_info{'overlap'} = $overlap;
	$search_info{'motif_name'} = $motif_name;
	$search_info{'motif_type'} = $motif_type;

	$mysql_info{'sql_user'} = $sql_user;
	$mysql_info{'sql_pass'} = $sql_pass;
	$mysql_info{'sql_db'} = $sql_db;

	SumSql("SELECT Sequence from $background order by ID", \@ID0, $hash_seq_ptr, $stat_ptr, \%search_info, $sequence_length, \%mysql_info);
	if ($number_of_inputs > 0) { 
		ReadClusters($cluster_input1, 1, \@ID1, $hash_seq_ptr, $stat_ptr, \%search_info, $sequence_length, $output); 
	} if ($number_of_inputs > 1) {
		ReadClusters($cluster_input2, 2, \@ID2, $hash_seq_ptr, $stat_ptr, \%search_info, $sequence_length, $output); 
	}

	for (my($i) = 0; $i <= ($number_of_inputs); ++$i) {
		if ($i == 0) { PickMotifs(\@ID0, \%BootstrapSums, $number_of_samples, $number_of_genes); @ID0 = (); $output_ptr = \@output_array0; }
		if ($i == 1) { PickMotifs(\@ID1, \%BootstrapSums, $number_of_samples, $number_of_genes); @ID1 = (); $output_ptr = \@output_array1; }
		if ($i == 2) { PickMotifs(\@ID2, \%BootstrapSums, $number_of_samples, $number_of_genes); @ID2 = (); $output_ptr = \@output_array2; }
		PrintStatistics($i, $stat_ptr, $sequence_length, $number_of_samples, $number_of_genes, $output_ptr, \%BootstrapSums);
	}

	PrintIntoOneFile($number_of_inputs, \@output_array0, \@output_array1, \@output_array2, $output, $motif_name);
	DrawGnuPlot($number_of_inputs, $stat_ptr, $output, $motif_name, $number_of_samples, $number_of_genes, $sequence_length);
	return(1);
}
###############################################################################################################
sub SumSql {
        my($haku, $array_ptr, $hash_seq_ptr, $stat_ptr, $search_info_ptr, $sequence_length, $mysql_info_ptr) = @_;
	my($count) = 0;											#Number of promoters total in the query
	my($match) = 0;											#Number of promoters having the motif
	my($TotSum) = 0;										#Total sum of counts

	my($hashlen) = ${$search_info_ptr}{'hashlen'};
	my($search_motif) = ${$search_info_ptr}{'search_motif'};
	my($overlap) = ${$search_info_ptr}{'overlap'};
	my($motif_name) = ${$search_info_ptr}{'motif_name'};
	my($motif_type) = ${$search_info_ptr}{'motif_type'};

	my($sql_user) = ${$mysql_info_ptr}{'sql_user'};
        my($sql_pass) = ${$mysql_info_ptr}{'sql_pass'};
        my($sql_db) = ${$mysql_info_ptr}{'sql_db'};

        my($sock) = "/var/lib/mysql/mysql.sock";                              # correction done 28.5.2008
        my($dsn) = "DBI:mysql:$sql_db;mysql_socket=$sock";
        my($dbh) = DBI->connect($dsn, $sql_user, $sql_pass);
        my($sth) = "";
	$sth = $dbh->prepare("$haku");
        $sth->execute();
        
	while (my $ref = $sth->fetchrow_hashref()) {
		my($seq) = "";
		my($sum) = 0;
		$seq = ($ref->{'Sequence'});

		if ($overlap == 0 && $motif_type == 1) { $sum = StringCountNoOverlap($seq, 0, $search_motif, $sequence_length ,""); }  
		if ($overlap == 1 && $motif_type == 1) { $sum = StringCountOverlap  ($seq, 0, $search_motif, $sequence_length, ""); }    
		if ($overlap == 0 && $motif_type == 2) { $sum = HashCountNoOverlap($seq, 0, $sequence_length, $hashlen, "", $hash_seq_ptr); }  
                if ($overlap == 1 && $motif_type == 2) { $sum = HashCountOverlap  ($seq, 0, $sequence_length, $hashlen, "", $hash_seq_ptr); } 

                push (@$array_ptr, $sum);
                if ($sum > 0) { $match = $match + 1}
                $count = $count + 1;
        	$TotSum = $TotSum + $sum;
	}

	@$stat_ptr[0] = $count;
        @$stat_ptr[1] = $match;
	if ($count > 0) {
		if (($match / $count) >= 0.1) { @$stat_ptr[2] = "TRUE"; } 
		else { @$stat_ptr[2] = "FALSE"; }
	}
        @$stat_ptr[3] = $TotSum;
	$dbh->disconnect();
}
#################################################################################################################
sub ReadClusters {
        my($cluster_file, $num, $array_ptr, $hash_seq_ptr, $stat_ptr, $search_info_ptr, $sequence_length, $output) = @_;
	my(@clusterlines)						= ();	#TMP-FILE WHICH CONTAINS SEQUENCES
	my(@sequences)							= ();	#SEQUENCES
	my(@genenames)							= ();	#HEADERS

	my($cluster) 							= "";	#CONTENT OF THE TMP-FILE
        my($count) 							= 0;	#
        my($match) 							= 0;	#
        my($TotSum) 							= 0;	#TOTAL NUMBER OF PATTERNS
        my($line) 							= "";	#USED IN LOOPS TO READ FILES
	my($fastanum) 							= -1;	#IS LINE SEQUENCE OR NOT
	my($id) 							= "";	#FASTA HEADER
	my($seq_true_len) 						= 0;	#THE TRUE LENGTH OF THE SEQUENCE

        my($hashlen) = ${$search_info_ptr}{'hashlen'};
        my($search_motif) = ${$search_info_ptr}{'search_motif'};
        my($overlap) = ${$search_info_ptr}{'overlap'};
        my($motif_name) = ${$search_info_ptr}{'motif_name'};
        my($motif_type) = ${$search_info_ptr}{'motif_type'};

	open(IN, "<$cluster_file") or die "could not open file $cluster_file\n"; 
	for(<IN>) { $cluster .= $_;}; 
	close(IN);

	$fastanum = &ReadFiles(\@sequences, \@genenames, $cluster);
	
	open (FP, ">>$output.out");
	print FP "\n#Found patterns ($motif_name) in cluster$num: (the report is in the gff-format)\n";
	print FP "#<seqname><source><feature><start><end><score><strand><frame>\[<pattern><input length>]\n";
	if ($fastanum == -1) {
		print FP "\n#INPUT DID NOT CONTAINED FASTA FORMATTED SEQUENCES, PLEASE CHECK THE FORMAT AND RESUBMIT THE JOB\n";
	}

        for(my($i) = 0; $i < @sequences; ++$i) {
		my($sum) = 0;
		$id = $genenames[$i];
		$id =~ s/\r|\n//g;
		$line = $sequences[$i];

                if ($overlap == 0 && $motif_type == 1) { $sum = StringCountNoOverlap($line, 1, $search_motif, $sequence_length, $id); }
                if ($overlap == 1 && $motif_type == 1) { $sum = StringCountOverlap  ($line, 1, $search_motif, $sequence_length, $id); }
                if ($overlap == 0 && $motif_type == 2) { $sum = HashCountNoOverlap($line, 1, $sequence_length, $hashlen, $id, $hash_seq_ptr); }
                if ($overlap == 1 && $motif_type == 2) { $sum = HashCountOverlap  ($line, 1, $sequence_length, $hashlen, $id, $hash_seq_ptr); }

                push (@$array_ptr, $sum);
                if ($sum > 0) { $match = $match + 1}
                $count = $count + 1;
                $TotSum = $TotSum + $sum;
        }
        close FP;

        @$stat_ptr[$num*14+0] = $count;
        @$stat_ptr[$num*14+1] = $match;
        if ($count > 0) {
		if (($match / $count) >= 0.1) { @$stat_ptr[$num*14+2] = "TRUE"; }
	        else { @$stat_ptr[$num*14+2] = "FALSE"; }
	}
        @$stat_ptr[$num*14+3] = $TotSum;
}
###############################################################################################################
sub PickMotifs {
	my($array_ptr, $bootstrap_ptr, $number_of_samples, $number_of_genes) = @_;
	%{$bootstrap_ptr} = ();
	for (my($i) = 0; $i < $number_of_samples; ++$i) {
		my($clsum) = 0;
		for (my ($j) = 0; $j < $number_of_genes ; ++$j) {		
	                my($random) = @$array_ptr[int( rand( scalar @$array_ptr ) )];
			$clsum = ($clsum + $random);	
		}
		if (exists ${$bootstrap_ptr}{$clsum}) {${$bootstrap_ptr}{$clsum} = (${$bootstrap_ptr}{$clsum}) + 1 ;}
		else {${$bootstrap_ptr}{$clsum} = 1} 
	}
}
############################################################################################################
sub PrintStatistics {  
        my($num, $stat_ptr, $sequence_length, $number_of_samples, $number_of_genes, $output_array, $bootstrap_ptr) = @_;
        my($median) = 0;
        my($modeV) = 0;
        my($modeK) = 0;
        my($stdev) = 0;
        my($stdsum) = 0;
        my($sum) = 0;
        my(@medianA) = ();
        my($medianK) = 0;
        my($medianV) = 0;
        my($leng_avg) = 0;
        my($conf95) = 0;
        my($conf99) = 0;
	
        for my $key (sort { $a <=> $b } keys %{$bootstrap_ptr} ) {
                my ($value) = ${$bootstrap_ptr}{$key};
		push(@$output_array, [$key, $value]);
                $stdsum = $stdsum + (($key**2)*$value);
                $sum = $sum + ($value * $key);
                push (@medianA, $key);
                if ($modeV < $value) {$modeV = $value; $modeK = $key;}
	}

        $medianK = splice @medianA, ((scalar @medianA/2)-1),1;
        $medianV = ${$bootstrap_ptr}{$medianK};

        $stdev = ($stdsum - ((1/$number_of_samples)*($sum**2)));
	$stdev = ((1/($number_of_samples-1)) * $stdev);
	if($stdev < 0 && $stdev > -0.000000001) { $stdev = 0; }
	$stdev = sprintf("%.2f", (sqrt($stdev)));
        $conf95 = (1.96*($stdev/(sqrt($number_of_samples))));
        $conf99 = (2.58*($stdev/(sqrt($number_of_samples))));

        @$stat_ptr[$num*14+4] = $sum;
        @$stat_ptr[$num*14+5] = "$modeK = $modeV";
        @$stat_ptr[$num*14+6] = "$medianK = $medianV";
        @$stat_ptr[$num*14+7] = $stdev;
        @$stat_ptr[$num*14+8] = sprintf("%.2f", ($sum / $number_of_samples));
        @$stat_ptr[$num*14+9] = sprintf("%.2f", (($sum / $number_of_samples) / $number_of_genes));
	if ($sum !=0) { 
		@$stat_ptr[$num*14+10] = sprintf("%.2f", ((($sequence_length * 2) / (($sum / $number_of_samples) / $number_of_genes))));
	}
        @$stat_ptr[$num*14+11] = sprintf("%.2f", ($conf95));
	@$stat_ptr[$num*14+12] = sprintf("%.2f", ($conf99));
        @$stat_ptr[$num*14+13] = $stdsum;
}
############################################################################################################
sub DrawGnuPlot {
        my($num, $stat_ptr, $output, $motif_name, $number_of_samples, $number_of_genes, $sequence_length) = @_;
        my($plot_cmd) = "";
        my($xlabel) = "'No. of occurrences of ".$motif_name." in pseudo-cluster'";
        my($ylabel) = "'No. of pseudo-clusters having given occurrence of ".$motif_name."'";
        my($title) = "'Analysis of ".$motif_name." pattern: ".$number_of_samples." pseudo-clusters of ".$number_of_genes." promoters (length = ".$sequence_length." bp)'\n";
        open(GP, "|gnuplot");
        print GP "set terminal png small\n";
        print GP "set data style line\n";
        print GP "set title $title\n";
        print GP "set xlabel $xlabel\n";
        print GP "set ylabel $ylabel\n";
        print GP "set output '".$output.".png'\n";

	$plot_cmd ="plot ";
	
        if ($num > -1 && @$stat_ptr[7] > 0) {
                $plot_cmd .= "'$output\.out' u 1:2 smooth bezier title 'Background' w lines 1, ";
		$plot_cmd .= "'$output\.out' u 1:2 title 'Background' w lines 6";
        }
        if ($num >  0 && @$stat_ptr[21] > 0) {
		if(length($plot_cmd) > 5) { $plot_cmd .= ", "; }
		
		$plot_cmd .= "'$output\.out' u 3:4 smooth bezier title 'Cluster1' w lines 8, ";
		$plot_cmd .= "'$output\.out' u 3:4 title 'Cluster1' w lines 3";
        }
        if ($num >  1 && @$stat_ptr[35] > 0) {
		if(length($plot_cmd) > 5) { $plot_cmd .= ", "; }

                $plot_cmd .= "'$output\.out' u 5:6 smooth bezier title 'Cluster2' w lines 2, ";
		$plot_cmd .= "'$output\.out' u 5:6 title 'Cluster2' w lines 7";
        }
        print GP "$plot_cmd\n";

        if ((@$stat_ptr[7] == 0) && (@$stat_ptr[21] == 0) && (@$stat_ptr[35] == 0)) {
                print GP "set yrange[0:1]\n";
                print GP "set xrange[0:1]\n";
                print GP "plot 0\n";
        }

        close GP;
}
############################################################################################################
sub PrintIntoOneFile {
	my($num, $array0_ptr, $array1_ptr, $array2_ptr, $output, $motif_name) = @_;
	my($max) = 0;
	my($text) = "";
	
	$max = scalar(@$array0_ptr);
	if ($max < scalar(@$array1_ptr)) { $max = scalar(@$array1_ptr); }
	if ($max < scalar(@$array2_ptr)) { $max = scalar(@$array2_ptr); }

	open (FF, ">>$output.out");
        print FF "\n#Bootstrap results: first column is the number of occurrences of the pattern ($motif_name)\n";  
	print FF "#within one artificial cluster. second column is the number of the artificial clusters having\n";
        print FF "#the pointed preciding occurrence. (For example, value 60\t13, means that there are 13 artificial\n";
        print FF "#clusters that have the pattern X 60 times.\n";

	print FF "\n#BACKGROUND\t\tCLUSTER 1\t\tCLUSTER 2\n";
	for (my($i) = 0 ; $i < $max; ++$i) {
                if ($i < scalar(@$array0_ptr)) { $text =  "@$array0_ptr[$i]->[0]\t@$array0_ptr[$i]->[1]\t"; }
                else { $text = "\$\t\$\t"; }
                if ($i < scalar(@$array1_ptr)) { $text .= "@$array1_ptr[$i]->[0]\t@$array1_ptr[$i]->[1]\t"; }
                else { $text.= "\$\t\$\t"; }
                if ($i < scalar(@$array2_ptr)) { $text .= "@$array2_ptr[$i]->[0]\t@$array2_ptr[$i]->[1]"; }
                else { $text.= "\$\t\$\t"; }
                print FF "$text\n";
        }
        close(FF);
}
############################################################################################################
1;
