#! /usr/bin/perl
use warnings;
use strict;
use FindBin qw($Bin);
use Getopt::Long;

use lib "$Bin";
use PromoBootstrap;
use PoboStatistics; 
use FuzzyConverter;
use FuzzyConverter;
use ReadFiles;

########################################################################################################################################
# command line parsing part

my %opt = ();
my %legend = (
           "input_num=i"	=>      "are we using 1 or 2 clusters (value 1 or 2)",
           "gene_file1=s"	=>      "filename with sequence from cluster 1",
           "gene_file2=s"	=>      "filename with sequence from cluster 2",
           "promo_num=i"	=>      "number of repeats",
           "clust_num=i"	=>      "number of clusters",
           "search_mot=s"	=>      "motif to search for (EMPTY IF MATRIX USED)",
           "promo_len=i"	=>      "Lenght of the sequence",
           "background=s"	=>      "Chosen background file for MySql",
           "motiftype=s"	=>	"Is the motif a string or a matirx",
           "search_mat=s"	=>      "filename with matrix from matirx filed",
           "cutoff_mat=s"	=>      "Cut-off for the matrix",
	   "overlap=s"		=>	"Overlapping pattern",
	   "sql_user=s"		=>	"mysql user",
	   "sql_pass=s"		=>	"mysql_password",
	   "sql_db=s"		=>	"mysql database",
	   "output=s"		=>	"outfile", 
	   "calladd=s"		=>	"call address",
);

GetOptions(\%opt, "help!", keys %legend);

my($input_num)   = "";		#are we using 1 or 2 clusters (value 1 or 2)
my($file_gene1)  = "";		#sequence from cluster 1 (tmp file)
my($file_gene2)  = "";		#sequence from cluster 2 (tmp file)
my($promo_num)   = "";		#number of repeats
my($clust_num)   = "";		#number of clusters
my($search_mot)  = "";		#motif to search for (EMPTY IF MATRIX USED)
my($promo_len)   = "";		#lenght of the promoter
my($background)  = "";		#number of the background (use in MYSQL statement)
my($mat_or_pat)  = "";		#is the motif a string or matrix (value 1 or 2)
my($search_mat)  = "";		#the matrix (tmp file)
my($cutoff_mat)  = "";		#cut-off value for the matrix (EMPTY IF CONSENSUS USED)
my($overlap)     = "";		#Overlaping paterns
my($sql_user)    = "";		#Mysql username
my($sql_pass)    = "";		#Mysql password
my($sql_db)      = "";		#Mysql database name
my($output)      = "";		#output file
my($calladd)	 = "";		#calling address
my($fastanum1)	 = "";		#number fasta seqs
my($fastanum2)   = "";		#number fasta seqs

my(@temp_seqs)	 = ();
my(@temp_heads)	 = ();

if (exists($opt{'input_num'}))	{ $input_num = $opt{'input_num'}; }	else { die "Missing parameter: input_num"; } 
if (exists($opt{'gene_file1'}))	{ $file_gene1 = $opt{'gene_file1'};}	else { die "Missing parameter: gene_file1"; } 

if (exists($opt{'promo_num'}))	{ $promo_num = $opt{'promo_num'};}	else { die "Missing parameter: promo_num"; }
if (exists($opt{'clust_num'}))	{ $clust_num = $opt{'clust_num'};}	else { die "Missing parameter: clust_num"; }
if (exists($opt{'promo_len'}))	{ $promo_len  = $opt{'promo_len'};}	else { die "Missing parameter: promo_len"; }
if (exists($opt{'background'}))	{ $background = $opt{'background'};}	else { die "Missing parameter: background"; }
if (exists($opt{'motiftype'}))	{ $mat_or_pat = $opt{'motiftype'};}	else { die "Missing parameter: motiftype"; }
if (exists($opt{'overlap'}))	{ $overlap = $opt{'overlap'}; }		else { die "Missing parameter: overlap"; }
if (exists($opt{'sql_user'}))	{ $sql_user = $opt{'sql_user'}; }	else { die "Missing parameter: sql_user"; }
if (exists($opt{'sql_pass'}))	{ $sql_pass = $opt{'sql_pass'}; }	else { die "Missing parameter: sql_pass"; }
if (exists($opt{'sql_db'}))	{ $sql_db = $opt{'sql_db'}; }		else { die "Missing parameter: sql_db"; }
if (exists($opt{'output'}))	{ $output = $opt{'output'}; }		else { die "Missing parameter: output"; }
if (exists($opt{'calladd'}))    { $calladd = $opt{'calladd'}; }         else { die "Missing parameter: calladd"; }

if($input_num == 2) {
	if (exists($opt{'gene_file2'})) { $file_gene2 = $opt{'gene_file2'};}    else { die "Missing parameter: gene_file2"; }
} else {
	if (exists($opt{'gene_file2'})) { $file_gene2 = $opt{'gene_file2'};}    else { $file_gene2 = "null"; }
}
if ($mat_or_pat == 1) {
	if (exists($opt{'search_mot'})) { $search_mot = $opt{'search_mot'};}    else { die "Missing parameter: search_mot"; }
	if (exists($opt{'search_mat'})) { $search_mat = $opt{'search_mat'};}    else { $search_mat = "null"; }
	if (exists($opt{'cutoff_mat'})) { $cutoff_mat = $opt{'cutoff_mat'};}    else { $cutoff_mat = "null"; }
} elsif ($mat_or_pat ==2) {
	if (exists($opt{'search_mot'})) { $search_mot = $opt{'search_mot'};}    else { $search_mot = "null"; }
	if (exists($opt{'search_mat'})) { $search_mat = $opt{'search_mat'};}    else { die "Missing parameter: search_mat"; }
	if (exists($opt{'cutoff_mat'})) { $cutoff_mat = $opt{'cutoff_mat'};}    else { die "Missing parameter: cutoff_mot"; }
} elsif ($mat_or_pat ==3) {
	if (exists($opt{'search_mot'})) { $search_mot = $opt{'search_mot'};}    else { die "Missing parameter: search_mot"; }
	if (exists($opt{'search_mat'})) { $search_mat = $opt{'search_mat'};}    else { die "Missing parameter: search_mat"; }
	if (exists($opt{'cutoff_mat'})) { $cutoff_mat = $opt{'cutoff_mat'};}    else { die "Missing parameter: cutoff_mot"; }
}

########################################################################################################################################
# MODULES OWN PARAMETERS

my(@anova_table) 		= ();
my(@ttest_table) 		= ();
my(@stat_array) 		= ();
my(%matrix_hash) 		= ();
my($return_value)		= 0;
my($matrix_patterns)		= "";
my($result_file)		= "";
my($fuzzy_len)			= 0;
my($search_fuzzy)		= "";
my($stat_text)			= "";

my($open_tag) 			= "";
my($close_tag) 			= "";

my($input_gene1_seq) 		= "";
my($input_gene2_seq) 		= "";
my($matrix_counter) 		= 0;
my(@file_list)			= ($file_gene1, $file_gene2, $search_mat);

########################################################################################################################################
# CONVERTS FUZZY TO MATRIX

if ($mat_or_pat == 3) {
	$search_fuzzy = &Convertfuzzytomatrix($search_mot);
	$fuzzy_len = length($search_mot);
	$cutoff_mat = ($fuzzy_len - $cutoff_mat); 
	$mat_or_pat = 2;
	open (MTRX, ">$search_mat");
	print MTRX "$search_fuzzy";
	close MTRX;
}
########################################################################################################################################
# CALLER (file is for web-tools to link the server file to correct html-folder !!!)

$return_value = MainProgram($file_gene1, $file_gene2, $input_num, $promo_num, $clust_num, $search_mot, $promo_len, $background,
                   $mat_or_pat, $search_mat, $cutoff_mat, $overlap, $sql_user, $sql_pass, $sql_db, $output, \@stat_array, \%matrix_hash);

$open_tag = "\<td align='center'\>\<font color = '#000000'\>";
$close_tag = "\<\/font\>\<\/td\>\n";

$result_file = ">".$output."\.html";
open(OUTPUT, $result_file);

if($mat_or_pat == 2) {
	open(IN, "<$search_mat") or die "could not open file $search_mat\n";
	$search_mot = <IN>;
	close IN;
	$search_mot =~ s/\n|\r//gi;
}

########################################################################################################################################
# HTML OUTPUT PRINTING

print OUTPUT "<dtml-var pobo_header>\n";
print OUTPUT "<dtml-var result_header>\n";
print OUTPUT "<img bgcolor = '#000000' src='$calladd\.png'><br><br>\n";

########################################################################################################################################
# GENERAL INFORMATION TABLE 1

print OUTPUT "<table border='1' cellpadding = '4' cellspacing = '0' align = 'center' bordercolor = '#00008B' bgcolor = '#B0C4DE' width='95%'>\n";
print OUTPUT "<tr>\n";
print OUTPUT "<td align='center' colspan ='5'><font color = '#000000'><b>Input information</b></font></td>\n";
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT "<td align='center'><a href='helpR#p1' target ='_blank'>Searched pattern</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p2' target ='_blank'>Background model</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p3' target ='_blank'>Number of promoters in a cluster</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p4' target ='_blank'>Number of pseudo-clusters</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p5' target ='_blank'>Length of promoters</a></td>\n";
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag.$search_mot.$close_tag;
print OUTPUT $open_tag.$background.$close_tag;
print OUTPUT $open_tag.$promo_num.$close_tag;
print OUTPUT $open_tag.$clust_num.$close_tag;
print OUTPUT $open_tag.$promo_len.$close_tag;
print OUTPUT "</tr>\n</table>\n<br>\n";

########################################################################################################################################
# GENERAL INFORMATION TABLE 2

print OUTPUT "<table border='1' cellpadding = '4' cellspacing = '0' align = 'center' bordercolor = '#00008B' bgcolor = '#B0C4DE' width='95%'>\n";
print OUTPUT "<tr>\n";
print OUTPUT "<td align='center' colspan ='5'><font color = '#000000'><b>General information about the clusters</b></font></td>\n";
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT "<td align='center'><a href='helpR#p6' target ='_blank'>Searched dataset</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p7' target ='_blank'>Number of promoters in a dataset</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p8' target ='_blank'>Number of promoters containing the pattern</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p9' target ='_blank'>10% co-expression rule</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p10' target ='_blank'>Number of patterns totally in a dataset</a></td>\n";
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag."Background".$close_tag;
print OUTPUT $open_tag.$stat_array[0].$close_tag;
print OUTPUT $open_tag.$stat_array[1].$close_tag;
print OUTPUT $open_tag.$stat_array[2].$close_tag;
print OUTPUT $open_tag.$stat_array[3].$close_tag;
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag."Cluster 1".$close_tag;
print OUTPUT $open_tag.$stat_array[14].$close_tag; 
print OUTPUT $open_tag.$stat_array[15].$close_tag;
print OUTPUT $open_tag.$stat_array[16].$close_tag;
print OUTPUT $open_tag.$stat_array[17].$close_tag;
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag."Cluster 2".$close_tag;
print OUTPUT $open_tag.$stat_array[28].$close_tag;
print OUTPUT $open_tag.$stat_array[29].$close_tag;
print OUTPUT $open_tag.$stat_array[30].$close_tag;
print OUTPUT $open_tag.$stat_array[31].$close_tag;
print OUTPUT "</tr>\n</table>\n<br>\n";

########################################################################################################################################
# BASIC STAT TABLE

print OUTPUT "<table border='1' cellpadding = '4' cellspacing = '0' align = 'center' bordercolor = '#00008B' bgcolor = '#B0C4DE' width='95%'>\n";
print OUTPUT "<tr>\n";
print OUTPUT "<td align='center' colspan ='10'><font color = '#000000'><b>Basic statistics</b></font></td>\n";
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT "<td align='center'><a href='helpR#p6' target ='_blank'>Searched dataset</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p12' target ='_blank'>Total patterns</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p13' target ='_blank'>Cluster mean</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p14' target ='_blank'>Promoter mean</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p15' target ='_blank'>Pattern distribution</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p16' target ='_blank'>StdDev</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p17' target ='_blank'>Mode</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p18' target ='_blank'>Median</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p19' target ='_blank'>99% Confidence</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p20' target ='_blank'>95% Confidence</a></td>\n";
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag."Background".$close_tag;
print OUTPUT $open_tag.$stat_array[4].$close_tag;
print OUTPUT $open_tag.$stat_array[8].$close_tag;
print OUTPUT $open_tag.$stat_array[9].$close_tag;
print OUTPUT $open_tag.$stat_array[10].$close_tag;
print OUTPUT $open_tag.$stat_array[7].$close_tag;
print OUTPUT $open_tag.$stat_array[5].$close_tag;
print OUTPUT $open_tag.$stat_array[6].$close_tag;
print OUTPUT $open_tag.$stat_array[11].$close_tag;
print OUTPUT $open_tag.$stat_array[12].$close_tag;
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag."Cluster 1".$close_tag;
print OUTPUT $open_tag.$stat_array[18].$close_tag;
print OUTPUT $open_tag.$stat_array[22].$close_tag;
print OUTPUT $open_tag.$stat_array[23].$close_tag;
print OUTPUT $open_tag.$stat_array[24].$close_tag;
print OUTPUT $open_tag.$stat_array[21].$close_tag;
print OUTPUT $open_tag.$stat_array[19].$close_tag;
print OUTPUT $open_tag.$stat_array[20].$close_tag;
print OUTPUT $open_tag.$stat_array[25].$close_tag;
print OUTPUT $open_tag.$stat_array[26].$close_tag;
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag."Cluster 2".$close_tag;
print OUTPUT $open_tag.$stat_array[32].$close_tag;
print OUTPUT $open_tag.$stat_array[36].$close_tag;
print OUTPUT $open_tag.$stat_array[37].$close_tag; 
print OUTPUT $open_tag.$stat_array[38].$close_tag;
print OUTPUT $open_tag.$stat_array[35].$close_tag;
print OUTPUT $open_tag.$stat_array[33].$close_tag;
print OUTPUT $open_tag.$stat_array[34].$close_tag;
print OUTPUT $open_tag.$stat_array[39].$close_tag;
print OUTPUT $open_tag.$stat_array[40].$close_tag;
print OUTPUT "</tr>\n</table>\n<br>";

########################################################################################################################################
# T-TEST

&CalculateTtest($input_num, $clust_num, \@stat_array, \@ttest_table);

########################################################################################################################################
# T-TEST PRINTING

print OUTPUT "<table border='1' cellpadding = '4' cellspacing = '0' align = 'center' bordercolor = '#00008B' bgcolor = '#B0C4DE' width='95%'>\n";
print OUTPUT "<tr>\n";
print OUTPUT "<td align='center' colspan ='3'><font color = '#000000'><b>Independent T-test statistics<b></font></td>\n";
print OUTPUT "</tr><tr>\n";
print OUTPUT "<td align='center'><a href='helpR#p26' target ='_blank'>t-Test</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p27' target ='_blank'>t-Value</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p23' target ='_blank'>Degrees of Freedom</a></td>\n";
print OUTPUT "</tr><tr>\n";
print OUTPUT $open_tag."Between Cluster 1 and background".$close_tag;
print OUTPUT $open_tag.$ttest_table[0].$close_tag;
print OUTPUT $open_tag.$ttest_table[1].$close_tag;
print OUTPUT "</tr><tr>\n";
print OUTPUT $open_tag."Between Cluster 2 and background".$close_tag;
print OUTPUT $open_tag.$ttest_table[2].$close_tag;
print OUTPUT $open_tag.$ttest_table[3].$close_tag;
print OUTPUT "</tr>\n<tr>";
print OUTPUT $open_tag."Between Cluster 2 and 1".$close_tag;
print OUTPUT $open_tag.$ttest_table[4].$close_tag;
print OUTPUT $open_tag.$ttest_table[5].$close_tag;
print OUTPUT "</tr></table></form><br>\n";

########################################################################################################################################
# ANOVA

&CalculateANOVA($input_num, $clust_num, \@stat_array, \@anova_table);

########################################################################################################################################
# ANOVA PRINTING

print OUTPUT "<table border='1' cellpadding = '4' cellspacing = '0' align = 'center' bordercolor = '#00008B' bgcolor = '#B0C4DE' width='95%'>\n";
print OUTPUT "<tr>\n";
print OUTPUT "<td align='center' colspan ='5'><font color = '#000000'><b>ANOVA-test (Analyses of Variances) statistics</b></font></td>\n";
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT "<td align='center'><a href='helpR#p21' target ='_blank'>ANOVA-test</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p22' target ='_blank'>Sums of Squares</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p23' target ='_blank'>Degrees of Freedom</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p24' target ='_blank'>Mean Sum of squares</a></td>\n";
print OUTPUT "<td align='center'><a href='helpR#p25' target ='_blank'>F-factor</a></td>\n";
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag."Between groups".$close_tag;
print OUTPUT $open_tag.$anova_table[0].$close_tag;
print OUTPUT $open_tag.$anova_table[1].$close_tag;
print OUTPUT $open_tag.$anova_table[2].$close_tag;
print OUTPUT $open_tag.$anova_table[3].$close_tag;
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag."Within groups".$close_tag;
print OUTPUT $open_tag.$anova_table[4].$close_tag;
print OUTPUT $open_tag.$anova_table[5].$close_tag;
print OUTPUT $open_tag.$anova_table[6].$close_tag;
print OUTPUT $open_tag."-".$close_tag;
print OUTPUT "</tr>\n<tr>\n";
print OUTPUT $open_tag."Total in groups".$close_tag;
print OUTPUT $open_tag.$anova_table[7].$close_tag;
print OUTPUT $open_tag.$anova_table[8].$close_tag;  
print OUTPUT $open_tag."-".$close_tag;
print OUTPUT $open_tag."-".$close_tag;
print OUTPUT "</tr>\n</table>\n<br>\n";

########################################################################################################################################
# DTML-JAVASCRIPT VARIABLE PRINTING

if(($input_num == 1) || ($input_num == 2)) {
	open(IN, "<$file_gene1") or die "could not open file $file_gene1\n"; 
	for(<IN>) { $input_gene1_seq .= $_;} 
	close(IN);
} 
if ($input_num == 2) {
	open(IN, "<$file_gene2") or die "could not open file $file_gene2\n"; 
	for(<IN>) { $input_gene2_seq .= $_;} 
	close(IN);
}

$fastanum1 = &ReadFiles(\@temp_seqs, \@temp_heads, $input_gene1_seq);
$fastanum2 = &ReadFiles(\@temp_seqs, \@temp_heads, $input_gene2_seq);
if ($fastanum1 == -1) { $input_gene1_seq = "";}
if ($fastanum2 == -1) { $input_gene2_seq = "";}

$input_gene1_seq =~ s/\'/ /gi;
$input_gene1_seq =~ s/\"/ /gi;
$input_gene1_seq =~ s/\r/ /gi;
$input_gene1_seq =~ s/\t/ /gi;

$input_gene2_seq =~ s/\'/ /gi;
$input_gene2_seq =~ s/\"/ /gi;
$input_gene2_seq =~ s/\r/ /gi;
$input_gene2_seq =~ s/\t/ /gi;

if($mat_or_pat == 2) { for my $key(keys(%matrix_hash)) { $matrix_patterns .= $key."\n"; } }	#create matrix

print OUTPUT "<form name=\"visualize\" method=\"post\" enctype=\"multipart/form-data\">\n";
print OUTPUT "<Input type=\"hidden\" name=\"caller\"			size=\"0\" value=\"pobo\">\n";
print OUTPUT "<Input type=\"hidden\" name=\"sequence_set1\"		size=\"0\" value=\"$input_gene1_seq\">\n";
print OUTPUT "<Input type=\"hidden\" name=\"sequence_set2\"		size=\"0\" value=\"$input_gene2_seq\">\n";
print OUTPUT "<Input type=\"hidden\" name=\"input_number\"		size=\"0\" value=\"$input_num\">\n";
print OUTPUT "<Input type=\"hidden\" name=\"sequence_length\"		size=\"0\" value=\"$promo_len\">\n";
print OUTPUT "<Input type=\"hidden\" name=\"pat_overlap\"		size=\"0\" value=\"$overlap\">\n";
print OUTPUT "<Input type=\"hidden\" name=\"background_organism\"       size=\"0\" value=\"$background\">\n";

if($mat_or_pat == 1) {
	print OUTPUT "<Input type=\"hidden\" name=\"search_pattern\" size=\"0\" value=\"$search_mot\">\n";
	print OUTPUT "<Input type=\"hidden\" name=\"group_me\" size=\"0\" value=\"0\">\n";
} else {
	print OUTPUT "<Input type=\"hidden\" name=\"search_pattern\" size=\"0\" value=\"$matrix_patterns\">\n";
	print OUTPUT "<Input type=\"hidden\" name=\"group_me\" size=\"0\" value=\"1\">\n";
}

$input_gene1_seq = "";
$input_gene2_seq = "";

########################################################################################################################################
# END TAGS

print OUTPUT "</form>\n";
print OUTPUT "<dtml-unless result_file><dtml-call \"REQUEST.set('result_file', '$calladd\.out')\"></dtml-unless>\n";
print OUTPUT "<dtml-var result_links>\n";
print OUTPUT "<dtml-var pobo_footer>\n";
unlink @file_list;

eval 'use lib "/data/backup/zope_data/poxo/COMMON"';
eval 'use CreateCheckfile';
eval '&CreateCheckfile("$output\.checkme")';

exit;
